# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['avstack',
 'avstack.ego',
 'avstack.environment',
 'avstack.geometry',
 'avstack.modules',
 'avstack.modules.control',
 'avstack.modules.fusion',
 'avstack.modules.localization',
 'avstack.modules.perception',
 'avstack.modules.planning',
 'avstack.modules.prediction',
 'avstack.modules.safety',
 'avstack.modules.tracking',
 'avstack.modules.tracking.libraries',
 'avstack.modules.tracking.libraries.AB3DMOT',
 'avstack.modules.tracking.libraries.EagerMOT',
 'avstack.utils']

package_data = \
{'': ['*']}

install_requires = \
['Pillow>=9,<10',
 'cython>=0.29,<0.30',
 'filterpy>=1,<2',
 'numba>=0.53.0,<0.54.0',
 'numpy-quaternion>=2022.4,<2023.0',
 'numpy>=1.19,<1.24',
 'opencv-python>=4.5.5,<5.0.0',
 'scikit-learn>=1.1,<2.0',
 'scipy>=1.9,<2.0',
 'tqdm>=4,<5']

setup_kwargs = {
    'name': 'lib-avstack-core',
    'version': '0.1.0a1',
    'description': 'Core library for autonomous vehicle development',
    'long_description': '# AVstack CORE\n\nThis is the core library of `AVstack`. It is independent of any dataset or simulator. This accompanies the preprint [HERE][avstack-preprint].\n\n## Philosophy\n\nPioneers of autonomous vehicles (AVs) promised to revolutionize the driving experience and driving safety. However, milestones in AVs have materialized slower than forecast. Two culprits are (1) the lack of verifiability of proposed state-of-the-art AV components, and (2) stagnation of pursuing next-level evaluations, e.g.,~vehicle-to-infrastructure (V2I) and multi-agent collaboration. In part, progress has been hampered by: the large volume of software in AVs, the multiple disparate conventions, the difficulty of testing across datasets and simulators, and the inflexibility of state-of-the-art AV components. To address these challenges, we present `AVstack`, an open-source, reconfigurable software platform for AV design, implementation, test, and analysis. `AVstack` solves the validation problem by enabling first-of-a-kind trade studies on datasets and physics-based simulators. `AVstack` solves the stagnation problem as a reconfigurable AV platform built on dozens of open-source AV components in a high-level programming language.\n\n\n## Installation\n\n**NOTE:** This currently only works on a Linux distribution (tested on Ubuntu 20.04). It also only works with Python 3.8 (to be expanded in the future).\n\nFirst, clone the repositry and submodules. If you are not running perception, then you may not have to recurse the submodules.\n```\ngit clone --recurse-submodules https://github.com/avstack-lab/lib-avstack-core.git \n```\nDependencies are managed with [`poetry`][poetry]. This uses the `pyproject.toml` file to create a `poetry.lock` file. It includes an optional `perception` group so that you can install `avstack` without all the large packages necessary for perception. To install poetry, see [this page](https://python-poetry.org/docs/#installation). \n\n\n### Without Perception\n\n**NOTE:** for now it is not possible to install without perception. We tried using poetry `groups` with the optional flag but there were unexpected consequences of being unable to install perception when used as a sub-project. This will hopefully be fixed soon.\n\n### With Perception\n\nFor installation with perception, run\n```\npoetry install\n```\n\n#### Perception Models\n\nWe integrate [mmlab](https://github.com/open-mmlab/)\'s `mmdet` and `mmdet3d` as third party submodules for perception. Running perception models requires a GPU! \n\nAt a minimum, you may want to run the provided unit tests. These require `mmdet` and `mmdet3d` perception models from the [`mmdet` model zoo][mmdet-modelzoo] and [`mmdet3d` model zoo][mmdet3d-modelzoo]. To do an automated installation of the necessary models, run:\n```\ncd models\n./download_mmdet_models.sh\n./download_mmdet3d_models.sh\n```\nThis will download the models to the `models` folder and will *attempt* to establish a symbolic link for `mmdet` and `mmdet3d`. We provide some error checking, but it is up to you to verify that the symbolic link worked.\n\n## Running Tests\n\nSince we are using `poetry`, run:\n```\npoetry run pytest tests\n```\nThese should pass if either: (a) you did not install perception, or (b) if you installed perception *and* downloaded the models. \n\n# Contributing\n\nSee [CONTRIBUTING.md][contributing] for further details.\n\n\n# LICENSE\n\nCopyright 2022 Spencer Hallyburton\n\nLicensed under the Apache License, Version 2.0 (the "License");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at\n\n    http://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an "AS IS" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.\n\nLogo by [Sydney Jeffs](https://twitter.com/sydney_jeffs).\n\n\n[avstack-preprint]: todo\n[poetry]: https://github.com/python-poetry/poetry\n[mmdet-modelzoo]: https://mmdetection.readthedocs.io/en/stable/model_zoo.html\n[mmdet3d-modelzoo]: https://mmdetection3d.readthedocs.io/en/stable/model_zoo.html\n[contributing]: https://github.com/avstack-lab/lib-avstack-core/blob/main/CONTRIBUTING.md\n[license]: https://github.com/avstack-lab/lib-avstack-core/blob/main/LICENSE.md\n',
    'author': 'Spencer Hallyburton',
    'author_email': 'spencer.hallyburton@duke.edu',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://avstack.org',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<3.9',
}


setup(**setup_kwargs)
